import { 
  Component, 
  OnInit, 
  ViewChild, 
  ElementRef, 
  AfterViewChecked 
} from '@angular/core';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

type SceneType = 'first-contact' | 'deep-talk';
type MessageRole = 'customer' | 'assistant';

interface Message {
  role: MessageRole;
  content: string;
  time: string;
}

interface Tag {
  text: string;
  icon: string;
  background: string;
  color: string;
}

interface Strategy {
  type: string;
  title: string;
  icon: string;
  iconBg: string;
  iconColor: string;
  description: string;
  highlight: string;
  applicable: string;
  advantage: string;
  successRate: string;
  rating: string;
}

@Component({
  selector: 'app-page-crm-decision',
  standalone: true,
  imports: [CommonModule, FormsModule],
  templateUrl: './page-crm-decision.html',
  styleUrls: ['./page-crm-decision.scss']
})
export class PageCrmDecision implements OnInit, AfterViewChecked {
  @ViewChild('chatContainer') chatContainer!: ElementRef;
  
  
  selectedScene: SceneType = 'first-contact';
  sceneTitle = '首次接触';
  
  messages: Message[] = [
    { role: 'customer', content: '您好，我对贵酒店的豪华套房很感兴趣，但觉得价格偏高，能否提供一些优惠？', time: '10:25 AM' },
    { role: 'assistant', content: '感谢您对我们豪华套房的关注！我们的套房拥有全景海景和私人管家服务，目前预订可享8折优惠，您觉得如何？', time: '10:26 AM' },
    { role: 'customer', content: '8折还是有点高，我看到其他酒店类似房型有更优惠的价格。', time: '10:27 AM' },
    { role: 'assistant', content: '我理解您的考虑。我们的套房包含双人早餐和免费SPA体验，这些增值服务能让您的住宿体验更加完美。', time: '10:28 AM' },
    { role: 'customer', content: '这个听起来不错，我考虑一下。', time: '10:29 AM' },
    { role: 'assistant', content: '好的，如果您今天预订，我们还可以额外赠送您一次下午茶体验。', time: '10:30 AM' }
  ];
  
  tags: Tag[] = [
    { text: 'VIP客户', icon: 'fas fa-user', background: 'rgba(100, 160, 255, 0.15)', color: '#64a0ff' },
    { text: '旺季时段', icon: 'fas fa-calendar', background: 'rgba(255, 212, 59, 0.15)', color: '#ffd43b' },
    { text: '价格敏感', icon: 'fas fa-percent', background: 'rgba(156, 54, 181, 0.15)', color: '#9c36b5' }
  ];
  
  currentRole: MessageRole = 'customer';
  newMessage = '';
  
  strategies: Strategy[] = [
    {
      type: 'aggressive',
      title: '主动型策略',
      icon: 'fas fa-bolt',
      iconBg: 'var(--blue-light)',
      iconColor: 'var(--blue)',
      description: '直接强调酒店独特价值，限量优惠制造紧迫感：',
      highlight: '',
      applicable: '客户表现出明确兴趣，决策周期短',
      advantage: '快速成交，提升单笔订单价值',
      successRate: '68%',
      rating: '⭐⭐⭐⭐'
    },
    {
      type: 'neutral',
      title: '平衡型策略',
      icon: 'fas fa-balance-scale',
      iconBg: 'rgba(255, 212, 59, 0.15)',
      iconColor: 'var(--yellow)',
      description: '提供阶梯式优惠，引导客户选择：',
      highlight: '',
      applicable: '客户在多个选项间犹豫',
      advantage: '平衡双方利益，建立长期关系',
      successRate: '82%',
      rating: '⭐⭐⭐⭐⭐'
    },
    {
      type: 'conservative',
      title: '保守型策略',
      icon: 'fas fa-shield-alt',
      iconBg: 'rgba(156, 54, 181, 0.15)',
      iconColor: 'var(--purple)',
      description: '提供灵活方案，降低决策风险：',
      highlight: '',
      applicable: '客户犹豫不决，决策周期长',
      advantage: '降低客户风险感知，提高转化率',
      successRate: '75%',
      rating: '⭐⭐⭐⭐'
    }
  ];
  
  activeCardIndex = 0;
  
  private strategiesContent: Record<SceneType, { aggressive: string; neutral: string; conservative: string }> = {
    "first-contact": {
      aggressive: "主动介绍酒店特色和限时优惠，强调独特卖点",
      neutral: "提供套餐选择，引导客户了解不同房型",
      conservative: "邀请客户参观虚拟酒店，提供详细资料"
    },
    "deep-talk": {
      aggressive: "深度挖掘客户需求，提供定制化解决方案",
      neutral: "建立信任关系，分享成功案例和客户评价",
      conservative: "提供专业建议，解答客户深层疑问"
    }
  };
  
  ngOnInit() {
    this.updateStrategies();
  }
  
  ngAfterViewChecked() {
    this.scrollToBottom();
  }
  
  scrollToBottom(): void {
    try {
      this.chatContainer.nativeElement.scrollTop = this.chatContainer.nativeElement.scrollHeight;
    } catch (err) {
      console.error(err);
    }
  }
  
  handleKeydown(event: KeyboardEvent): void {
    if (event.key === 'Enter' && !event.shiftKey) {
      event.preventDefault();
      this.sendMessage();
    }
  }
  onTextareaKeydown(event: KeyboardEvent): void {
    if (event.key === 'Enter' && !event.shiftKey) {
      event.preventDefault();
      this.sendMessage();
    }
  }
  
  updateStrategies(): void {
    const sceneStrategies = this.strategiesContent[this.selectedScene];
    this.strategies[0].highlight = sceneStrategies.aggressive;
    this.strategies[1].highlight = sceneStrategies.neutral;
    this.strategies[2].highlight = sceneStrategies.conservative;
  }
  
  selectScene(scene: SceneType): void {
    this.selectedScene = scene;
    this.sceneTitle = scene === 'first-contact' ? '首次接触' : '深度交流';
    this.updateStrategies();
  }
  
  setRole(role: MessageRole): void {
    this.currentRole = role;
  }
  
  addTag(): void {
    const newTag: Tag = {
      text: '新标签',
      icon: 'fas fa-tag',
      background: 'rgba(100, 160, 255, 0.15)',
      color: '#64a0ff'
    };
    this.tags.push(newTag);
  }
  
  removeTag(event: Event, tag: Tag): void {
    event.stopPropagation();
    this.tags = this.tags.filter(t => t !== tag);
  }
  
  sendMessage(): void {
    if (this.newMessage.trim()) {
      const now = new Date();
      const timeString = `${now.getHours()}:${now.getMinutes().toString().padStart(2, '0')}`;
      
      this.messages.push({
        role: this.currentRole,
        content: this.newMessage.trim(),
        time: timeString
      });
      
      this.newMessage = '';
      
      if (this.currentRole === 'assistant') {
        setTimeout(() => {
          this.simulateCustomerReply(now);
        }, 1500);
      }
    }
  }
  
  simulateCustomerReply(prevTime: Date): void {
    const replies = [
      "这个优惠包含早餐吗？",
      "我需要和家人商量一下",
      "能否提供免费接机服务？",
      "价格还是有点高，能再优惠些吗？",
      "你们的取消政策是怎样的？",
      "如果我今天预订，还有额外优惠吗？",
      "套餐包含哪些服务？",
      "有更经济的房型推荐吗？"
    ];
    
    const randomReply = replies[Math.floor(Math.random() * replies.length)];
    const time = new Date(prevTime.getTime() + 60000); // 加1分钟
    
    this.messages.push({
      role: 'customer',
      content: randomReply,
      time: `${time.getHours()}:${time.getMinutes().toString().padStart(2, '0')}`
    });
  }
  
  activateCard(index: number): void {
    this.activeCardIndex = index;
  }
  
  getCardTransform(index: number): string {
    if (index === this.activeCardIndex) {
      return 'translateY(0) scale(1)';
    } else if (Math.abs(index - this.activeCardIndex) === 1) {
      return `translateY(${20 * Math.abs(index - this.activeCardIndex)}px) scale(${1 - 0.05 * Math.abs(index - this.activeCardIndex)})`;
    } else {
      return `translateY(${40 * Math.abs(index - this.activeCardIndex)}px) scale(${1 - 0.1 * Math.abs(index - this.activeCardIndex)})`;
    }
  }
  
  getCardZIndex(index: number): string {
    if (index === this.activeCardIndex) {
      return '30';
    } else if (Math.abs(index - this.activeCardIndex) === 1) {
      return '20';
    } else {
      return '10';
    }
  }
  
  saveRecord(): void {
    console.log('记录已保存');
    // 这里可以添加实际的保存逻辑
  }
  
  viewHistory(): void {
    console.log('查看历史记录');
    // 这里可以添加历史查看逻辑
  }
}